==> babel-plugin-flow-react-proptypes-11.2.0.tgz: package/lib/index.js <==
'use strict';

var _util = require('./util');

var _convertToPropTypes = require('./convertToPropTypes');

var _convertToPropTypes2 = _interopRequireDefault(_convertToPropTypes);

var _makePropTypesAst = require('./makePropTypesAst');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// maps between type alias name to prop types
var internalTypes = {};

// maps between type alias to import alias
var importedTypes = {};

var exportedTypes = {};
var suppress = false;
var omitRuntimeTypeExport = false;

var SUPPRESS_STRING = 'no babel-plugin-flow-react-proptypes';

// General control flow:
// Parse flow type annotations in index.js
// Convert to intermediate representation via convertToPropTypes.js
// Convert to prop-types AST in makePropTypesAst.js

var convertNodeToPropTypes = function convertNodeToPropTypes(node) {
  return (0, _convertToPropTypes2.default)(node, importedTypes, internalTypes);
};

var getPropsForTypeAnnotation = function getPropsForTypeAnnotation(typeAnnotation) {
  var props = null;

  if (typeAnnotation.properties || typeAnnotation.type === 'GenericTypeAnnotation' || typeAnnotation.type === 'IntersectionTypeAnnotation' || typeAnnotation.type === 'AnyTypeAnnotation') {
    props = convertNodeToPropTypes(typeAnnotation);
  } else if (typeAnnotation.properties != null || typeAnnotation.type != null) {
    (0, _util.$debug)('typeAnnotation not of expected type, not generating propTypes: ', typeAnnotation);
  } else {
    throw new Error('Expected prop types, but found none. This is a bug in ' + _util.PLUGIN_NAME);
  }

  return props;
};

module.exports = function flowReactPropTypes(babel) {
  var t = babel.types;

  var opts = {};

  var isFunctionalReactComponent = function isFunctionalReactComponent(path) {
    if ((path.type === 'ArrowFunctionExpression' || path.type === 'FunctionExpression') && !path.parent.id) {
      // Could be functions inside a React component
      return false;
    }
    if ((0, _util.hasReactElementTypeAnnotationReturn)(path.node)) {
      return true;
    }
    if ((0, _util.containsReactElement)(path.node)) {
      return true;
    }
    return false;
  };

  /**
   * Adds propTypes or contextTypes annotations to code
   *
   * Extracts some shared logic from `annotate`.
   *
   * @param path
   * @param name
   * @param attribute - target member name ('propTypes' or 'contextTypes')
   * @param typesOrVar - propsOrVar / contextOrVar value
   */
  var addAnnotationsToAST = function addAnnotationsToAST(path, name, attribute, typesOrVar) {
    var attachPropTypesAST = void 0;
    // if type was exported, use the declared variable
    var valueNode = null;

    if (typeof typesOrVar === 'string') {
      valueNode = t.identifier(typesOrVar);
      attachPropTypesAST = t.expressionStatement(t.assignmentExpression('=', t.memberExpression(t.identifier(name), t.identifier(attribute)), valueNode));
    }
    // type was not exported, generate
    else {
        var propTypesAST = (0, _makePropTypesAst.makePropTypesAstForPropTypesAssignment)(typesOrVar);
        valueNode = propTypesAST;

        if (propTypesAST == null) {
          return;
        }
        attachPropTypesAST = t.expressionStatement(t.assignmentExpression('=', t.memberExpression(t.identifier(name), t.identifier(attribute)), propTypesAST));
      }

    if (opts.useStatic && (path.type === 'ClassDeclaration' || path.type === 'ClassExpression')) {
      var newNode = t.classProperty(t.identifier(attribute), valueNode);
      newNode.static = true;
      path.node.body.body.push(newNode);
    } else {
      path.insertAfter(attachPropTypesAST);
    }
  };

  /**
   * Called when visiting a node.
   *
   * Converts the props param to AST and attaches it at the proper location,
   * depending on the path param.
   *
   *
   * @param path
   * @param propsOrVar - props or exported props variable reference
   * @param contextOrVar - context or exported context variable reference
   */
  var annotate = function annotate(path, propsOrVar) {
    var contextOrVar = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

    var name = void 0;
    var targetPath = void 0;

    if (opts.useStatic && (path.type === 'ClassDeclaration' || path.type === 'ClassExpression')) {
      targetPath = path;
    } else if (path.type === 'ArrowFunctionExpression' || path.type === 'FunctionExpression') {
      name = path.parent.id.name;
      var basePath = path.parentPath.parentPath;
      targetPath = t.isExportDeclaration(basePath.parent) ? basePath.parentPath : basePath;
    } else if (path.node.id) {
      name = path.node.id.name;
      targetPath = ['Program', 'BlockStatement'].indexOf(path.parent.type) >= 0 ? path : path.parentPath;
    } else {
      throw new Error('babel-plugin-flow-react-proptypes attempted to add propTypes to a function/class with no name');
    }

    if (propsOrVar) {
      addAnnotationsToAST(targetPath, name, 'propTypes', propsOrVar);
    }

    if (contextOrVar) {
      addAnnotationsToAST(targetPath, name, 'contextTypes', contextOrVar);
    }
  };

  /**
   * Visitor for functions.
   *
   * Determines if a function is a functional react component and
   * inserts the proptypes and contexttypes AST via `annotate`.
   *
   * @param path
   */
  var functionVisitor = function functionVisitor(path) {
    if (!isFunctionalReactComponent(path)) {
      return;
    }

    // Check if this looks like a stateless react component with PropType reference:
    var firstParam = path.node.params[0];
    var typeAnnotation = firstParam && firstParam.typeAnnotation && firstParam.typeAnnotation.typeAnnotation;

    // Check if the component has context annotations
    var secondParam = path.node.params[1];
    var contextAnnotation = secondParam && secondParam.typeAnnotation && secondParam.typeAnnotation.typeAnnotation;

    var propsOrVar = null;
    if (!typeAnnotation) {
      (0, _util.$debug)('Found stateless component without type definition');
    } else {
      propsOrVar = typeAnnotation.id && exportedTypes[typeAnnotation.id.name] ? exportedTypes[typeAnnotation.id.name] : getPropsForTypeAnnotation(typeAnnotation);
    }

    var contextOrVar = void 0;

    if (contextAnnotation) {
      contextOrVar = contextAnnotation.id && exportedTypes[contextAnnotation.id.name] ? exportedTypes[contextAnnotation.id.name] : getPropsForTypeAnnotation(contextAnnotation);
    } else {
      contextOrVar = null;
    }

    if (propsOrVar) {
      annotate(path, propsOrVar, contextOrVar);
    }
  };

  return {
    visitor: {
      Program: function Program(path, _ref) {
        var _opts = _ref.opts;

        opts = _opts;
        internalTypes = {};
        importedTypes = {};
        exportedTypes = {};
        suppress = false;
        omitRuntimeTypeExport = opts.omitRuntimeTypeExport || false;
        var directives = path.node.directives;
        if (directives && directives.length) {
          var directive = directives[0];
          if (directive.value && directive.value.value == SUPPRESS_STRING) {
            suppress = true;
          }
        }
        if (this.file && this.file.opts && this.file.opts.filename) {
          if (this.file.opts.filename.indexOf('node_modules') >= 0) {
            // Suppress any file that lives in node_modules IF the
            // ignoreNodeModules setting is true
            suppress = opts.ignoreNodeModules;
          }
        }
      },
      TypeAlias: function TypeAlias(path) {
        if (suppress) return;
        (0, _util.$debug)('TypeAlias found');
        var right = path.node.right;


        var typeAliasName = path.node.id.name;
        if (!typeAliasName) {
          throw new Error('Did not find name for type alias');
        }

        var propTypes = convertNodeToPropTypes(right);
        internalTypes[typeAliasName] = propTypes;
      },
      "ClassExpression|ClassDeclaration": function ClassExpressionClassDeclaration(path) {
        if (!opts.useStatic && path.node.type === 'ClassExpression') return;

        if (path.node.BPFRP_HIT_FOR_CLASS_ANNOTATION) return;
        path.node.BPFRP_HIT_FOR_CLASS_ANNOTATION = true;

        if (suppress) return;
        var superClass = path.node.superClass;

        // check if we're extending React.Compoennt

        var extendsReactComponent = superClass && superClass.type === 'MemberExpression' && superClass.object.name === 'React' && (superClass.property.name === 'Component' || superClass.property.name === 'PureComponent');
        var extendsComponent = superClass && superClass.type === 'Identifier' && (superClass.name === 'Component' || superClass.name === 'PureComponent');
        if (!extendsReactComponent && !extendsComponent) {
          (0, _util.$debug)('Found a class that isn\'t a react component', superClass);
          return;
        }

        var propTypes = null,
            contextTypes = null;
        // And have type as property annotations
        path.node.body.body.forEach(function (bodyNode) {
          if (bodyNode && bodyNode.key.name === 'props' && bodyNode.typeAnnotation) {
            var annotation = bodyNode.typeAnnotation.typeAnnotation;
            var props = getPropsForTypeAnnotation(annotation);
            if (!props) {
              throw new TypeError('Couldn\'t process \`class { props: This }`');
            }

            propTypes = props;

            return;
          }

          if (bodyNode && bodyNode.key.name === 'context' && bodyNode.typeAnnotation) {
            var _annotation = bodyNode.typeAnnotation.typeAnnotation;
            var context = getPropsForTypeAnnotation(_annotation);
            if (!context) {
              throw new TypeError('Couldn\'t process \`class { context: This }`');
            }

            contextTypes = context;
          }
        });

        // or Component<void, Props, Context>
        var secondSuperParam = getPropsTypeParam(path.node);
        if (secondSuperParam && secondSuperParam.type === 'GenericTypeAnnotation') {
          var typeAliasName = secondSuperParam.id.name;
          if (typeAliasName === 'Object') return;
          var props = internalTypes[typeAliasName] || importedTypes[typeAliasName];
          if (!props) {
            throw new TypeError('Couldn\'t find type "' + typeAliasName + '"');
          }

          propTypes = props;
        }

        var thirdSuperParam = getContextTypeParam(path.node);
        if (thirdSuperParam && thirdSuperParam.type === 'GenericTypeAnnotation') {
          var _typeAliasName = thirdSuperParam.id.name;
          if (_typeAliasName === 'Object') return;
          var _props = internalTypes[_typeAliasName] || importedTypes[_typeAliasName];
          if (!_props) {
            throw new TypeError('Couldn\'t find type "' + _typeAliasName + '"');
          }

          contextTypes = _props;
        }

        annotate(path, propTypes, contextTypes);
      },
      FunctionExpression: function FunctionExpression(path) {
        if (suppress) return;
        return functionVisitor(path);
      },
      FunctionDeclaration: function FunctionDeclaration(path) {
        if (suppress) return;
        return functionVisitor(path);
      },
      ArrowFunctionExpression: function ArrowFunctionExpression(path) {
        if (suppress) return;
        return functionVisitor(path);
      },


      // See issue:
      /**
         * Processes exported type aliases.
         *
         * This function also adds something to the AST directly, instead
         * of invoking annotate.
         *
         * @param path
         * @constructor
         */
      ExportNamedDeclaration: function ExportNamedDeclaration(path) {
        if (suppress) return;
        var node = path.node;


        if (!node.declaration || node.declaration.type !== 'TypeAlias') {
          return;
        }

        var declarationObject = node.declaration.right;

        var name = node.declaration.id.name;
        var propTypes = convertNodeToPropTypes(declarationObject);
        internalTypes[name] = propTypes;

        var propTypesAst = (0, _makePropTypesAst.makePropTypesAstForExport)(propTypes);

        // create a variable for reuse
        var exportedName = (0, _util.getExportNameForType)(name);
        exportedTypes[name] = exportedName;
        var variableDeclarationAst = t.variableDeclaration('var', [t.variableDeclarator(t.identifier(exportedName), propTypesAst)]);
        path.insertBefore(variableDeclarationAst);

        if (!omitRuntimeTypeExport) {
          // add the variable to the exports
          var exportAst = t.expressionStatement(t.callExpression(t.memberExpression(t.identifier('Object'), t.identifier('defineProperty')), [t.identifier('exports'), t.stringLiteral((0, _util.getExportNameForType)(name)), t.objectExpression([t.objectProperty(t.identifier('value'), t.identifier(exportedName)), t.objectProperty(t.identifier('configurable'), t.booleanLiteral(true))])]));
          var conditionalExportsAst = t.ifStatement(t.binaryExpression('!==', t.unaryExpression('typeof', t.identifier('exports')), t.stringLiteral('undefined')), exportAst);
          path.insertAfter(conditionalExportsAst);
        }
      },
      ImportDeclaration: function ImportDeclaration(path) {
        if (suppress) return;
        var node = path.node;

        // https://github.com/brigand/babel-plugin-flow-react-proptypes/issues/62
        // if (node.source.value[0] !== '.') {
        //   return;
        // }

        if (node.importKind === 'type') {
          node.specifiers.forEach(function (specifier) {
            var typeName = specifier.type === 'ImportDefaultSpecifier' ? specifier.local.name : specifier.imported.name;
            // Store the name the type so we can use it later. We do
            // mark it as importedTypes because we do handle these
            // differently than internalTypes.
            // imported types are basically realized as imports;
            // because we can be somewhat sure that we generated
            // the proper exported propTypes in the imported file
            // Later, we will check importedTypes to determine if
            // we want to put this as a 'raw' type in our internal
            // representation
            importedTypes[typeName] = (0, _util.getExportNameForType)(typeName);

            // https://github.com/brigand/babel-plugin-flow-react-proptypes/issues/129
            if (node.source.value === 'react' && typeName === 'ComponentType') {
              var ast = t.variableDeclaration('var', [t.variableDeclarator(t.identifier((0, _util.getExportNameForType)(typeName)), t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral('prop-types')]), t.identifier('func')))]);
              path.insertAfter(ast);
              return;
            }

            var variableDeclarationAst = t.variableDeclaration('var', [t.variableDeclarator(
            // TODO: use local import name?
            t.identifier((0, _util.getExportNameForType)(typeName)), t.logicalExpression('||', t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral(node.source.value)]), t.identifier((0, _util.getExportNameForType)(typeName))), t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral('prop-types')]), t.identifier('any'))))]);
            path.insertAfter(variableDeclarationAst);
          });
        }
      }
    }
  };
};

function getPropsTypeParam(node) {
  if (!node) return null;
  if (!node.superTypeParameters) return null;
  var superTypes = node.superTypeParameters;
  if (superTypes.params.length === 2) {
    return superTypes.params[0];
  } else if (superTypes.params.length === 3) {
    return superTypes.params[0];
  } else if (superTypes.params.length === 1) {
    return superTypes.params[0];
  }
  return null;
}

function getContextTypeParam(node) {
  if (!node) return null;
  if (!node.superTypeParameters) return null;
  var superTypes = node.superTypeParameters;
  if (superTypes.params.length === 3) {
    return superTypes.params[2];
  }
  return null;
}

==> babel-plugin-flow-react-proptypes-12.1.0.tgz: package/lib/index.js <==
'use strict';

var _util = require('./util');

var _convertToPropTypes = require('./convertToPropTypes');

var _convertToPropTypes2 = _interopRequireDefault(_convertToPropTypes);

var _makePropTypesAst = require('./makePropTypesAst');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// maps between type alias name to prop types
var internalTypes = {};

// maps between type alias to import alias
var importedTypes = {};

var exportedTypes = {};
var suppress = false;
var omitRuntimeTypeExport = false;

var SUPPRESS_STRING = 'no babel-plugin-flow-react-proptypes';

// The template to use for the dead code elimination check
// if it passes, the prop types will be removed
// currently the 'deadCode' option is off by default, but this may change
var DEFAULT_DCE = 'process.env.NODE_ENV === \'production\'';

// General control flow:
// Parse flow type annotations in index.js
// Convert to intermediate representation via convertToPropTypes.js
// Convert to prop-types AST in makePropTypesAst.js

var convertNodeToPropTypes = function convertNodeToPropTypes(node) {
  return (0, _convertToPropTypes2.default)(node, importedTypes, internalTypes);
};

var getPropsForTypeAnnotation = function getPropsForTypeAnnotation(typeAnnotation) {
  var props = null;

  if (typeAnnotation.properties || typeAnnotation.type === 'GenericTypeAnnotation' || typeAnnotation.type === 'IntersectionTypeAnnotation' || typeAnnotation.type === 'AnyTypeAnnotation') {
    props = convertNodeToPropTypes(typeAnnotation);
  } else if (typeAnnotation.properties != null || typeAnnotation.type != null) {
    (0, _util.$debug)('typeAnnotation not of expected type, not generating propTypes: ', typeAnnotation);
  } else {
    throw new Error('Expected prop types, but found none. This is a bug in ' + _util.PLUGIN_NAME);
  }

  return props;
};

module.exports = function flowReactPropTypes(babel) {
  var t = babel.types;

  var opts = {};

  var _templateCache = {};
  function getDcePredicate() {
    // opts.deadCode could be a boolean (true for DEFAULT_DCE), or a string to be
    // used as a template
    // if it's falsy, then just return node without any wrapper
    if (!opts.deadCode) return null;

    // cache the template since it's going to be used a lot
    var templateCode = typeof opts.deadCode === 'string' ? opts.deadCode : DEFAULT_DCE;
    if (!_templateCache[templateCode]) {
      _templateCache[templateCode] = babel.template(templateCode);
    }

    // return a ternary
    var predicate = _templateCache[templateCode]({}).expression;
    return predicate;
  }

  function wrapInDceCheck(node) {
    var predicate = getDcePredicate(node);
    if (!predicate) return node;

    var conditional = t.conditionalExpression(predicate, t.nullLiteral(), node);

    return conditional;
  }

  var isFunctionalReactComponent = function isFunctionalReactComponent(path) {
    if ((path.type === 'ArrowFunctionExpression' || path.type === 'FunctionExpression') && !path.parent.id) {
      // Could be functions inside a React component
      return false;
    }
    if ((0, _util.hasReactElementTypeAnnotationReturn)(path.node)) {
      return true;
    }
    if ((0, _util.containsReactElement)(path.node)) {
      return true;
    }
    return false;
  };

  /**
   * Adds propTypes or contextTypes annotations to code
   *
   * Extracts some shared logic from `annotate`.
   *
   * @param path
   * @param name
   * @param attribute - target member name ('propTypes' or 'contextTypes')
   * @param typesOrVar - propsOrVar / contextOrVar value
   */
  var addAnnotationsToAST = function addAnnotationsToAST(path, name, attribute, typesOrVar) {
    var attachPropTypesAST = void 0;
    // if type was exported, use the declared variable
    var valueNode = null;

    if (typeof typesOrVar === 'string') {
      valueNode = t.identifier(typesOrVar);
      var inner = t.assignmentExpression('=', t.memberExpression(t.identifier(name), t.identifier(attribute)), valueNode);

      if (attribute === 'propTypes') {
        inner = wrapInDceCheck(inner);
      }

      attachPropTypesAST = t.expressionStatement(inner);
    }
    // type was not exported, generate
    else {
        var propTypesAST = (0, _makePropTypesAst.makePropTypesAstForPropTypesAssignment)(typesOrVar);
        valueNode = propTypesAST;

        if (attribute === 'propTypes') {
          valueNode = wrapInDceCheck(valueNode);
        }

        if (propTypesAST == null) {
          return;
        }
        attachPropTypesAST = t.expressionStatement(t.assignmentExpression('=', t.memberExpression(t.identifier(name), t.identifier(attribute)), valueNode));
      }

    if (opts.useStatic && (path.type === 'ClassDeclaration' || path.type === 'ClassExpression')) {
      var newNode = t.classProperty(t.identifier(attribute), valueNode);
      newNode.static = true;
      path.node.body.body.push(newNode);
    } else {
      path.insertAfter(attachPropTypesAST);
    }
  };

  /**
   * Called when visiting a node.
   *
   * Converts the props param to AST and attaches it at the proper location,
   * depending on the path param.
   *
   *
   * @param path
   * @param propsOrVar - props or exported props variable reference
   * @param contextOrVar - context or exported context variable reference
   */
  var annotate = function annotate(path, propsOrVar) {
    var contextOrVar = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

    var name = void 0;
    var targetPath = void 0;

    if (opts.useStatic && (path.type === 'ClassDeclaration' || path.type === 'ClassExpression')) {
      targetPath = path;
    } else if (path.type === 'ArrowFunctionExpression' || path.type === 'FunctionExpression') {
      name = path.parent.id.name;
      var basePath = path.parentPath.parentPath;
      targetPath = t.isExportDeclaration(basePath.parent) ? basePath.parentPath : basePath;
    } else if (path.node.id) {
      name = path.node.id.name;
      targetPath = ['Program', 'BlockStatement'].indexOf(path.parent.type) >= 0 ? path : path.parentPath;
    } else {
      throw new Error('babel-plugin-flow-react-proptypes attempted to add propTypes to a function/class with no name');
    }

    if (propsOrVar) {
      addAnnotationsToAST(targetPath, name, 'propTypes', propsOrVar);
    }

    if (contextOrVar) {
      addAnnotationsToAST(targetPath, name, 'contextTypes', contextOrVar);
    }
  };

  /**
   * Visitor for functions.
   *
   * Determines if a function is a functional react component and
   * inserts the proptypes and contexttypes AST via `annotate`.
   *
   * @param path
   */
  var functionVisitor = function functionVisitor(path) {
    if (!isFunctionalReactComponent(path)) {
      return;
    }

    // Check if this looks like a stateless react component with PropType reference:
    var firstParam = path.node.params[0];
    var typeAnnotation = firstParam && firstParam.typeAnnotation && firstParam.typeAnnotation.typeAnnotation;

    // Check if the component has context annotations
    var secondParam = path.node.params[1];
    var contextAnnotation = secondParam && secondParam.typeAnnotation && secondParam.typeAnnotation.typeAnnotation;

    var propsOrVar = null;
    if (!typeAnnotation) {
      (0, _util.$debug)('Found stateless component without type definition');
    } else {
      propsOrVar = typeAnnotation.id && exportedTypes[typeAnnotation.id.name] ? exportedTypes[typeAnnotation.id.name] : getPropsForTypeAnnotation(typeAnnotation);
    }

    var contextOrVar = void 0;

    if (contextAnnotation) {
      contextOrVar = contextAnnotation.id && exportedTypes[contextAnnotation.id.name] ? exportedTypes[contextAnnotation.id.name] : getPropsForTypeAnnotation(contextAnnotation);
    } else {
      contextOrVar = null;
    }

    if (propsOrVar) {
      annotate(path, propsOrVar, contextOrVar);
    }
  };

  return {
    visitor: {
      Program: function Program(path, _ref) {
        var _opts = _ref.opts;

        opts = _opts;
        internalTypes = {};
        importedTypes = {};
        exportedTypes = {};
        suppress = false;
        omitRuntimeTypeExport = opts.omitRuntimeTypeExport || false;
        var directives = path.node.directives;
        if (directives && directives.length) {
          var directive = directives[0];
          if (directive.value && directive.value.value == SUPPRESS_STRING) {
            suppress = true;
          }
        }
        if (this.file && this.file.opts && this.file.opts.filename) {
          if (this.file.opts.filename.indexOf('node_modules') >= 0) {
            // Suppress any file that lives in node_modules IF the
            // ignoreNodeModules setting is true
            suppress = opts.ignoreNodeModules;
          }
        }
      },
      TypeAlias: function TypeAlias(path) {
        if (suppress) return;
        (0, _util.$debug)('TypeAlias found');

        var typeAliasName = path.node.id.name;
        if (!typeAliasName) {
          throw new Error('Did not find name for type alias');
        }

        var propTypes = convertNodeToPropTypes(path.node);
        internalTypes[typeAliasName] = propTypes;
      },
      "ClassExpression|ClassDeclaration": function ClassExpressionClassDeclaration(path) {
        if (!opts.useStatic && path.node.type === 'ClassExpression') return;

        if (path.node.BPFRP_HIT_FOR_CLASS_ANNOTATION) return;
        path.node.BPFRP_HIT_FOR_CLASS_ANNOTATION = true;

        if (suppress) return;
        var superClass = path.node.superClass;

        // check if we're extending React.Compoennt

        var extendsReactComponent = superClass && superClass.type === 'MemberExpression' && superClass.object.name === 'React' && (superClass.property.name === 'Component' || superClass.property.name === 'PureComponent');
        var extendsComponent = superClass && superClass.type === 'Identifier' && (superClass.name === 'Component' || superClass.name === 'PureComponent');
        if (!extendsReactComponent && !extendsComponent) {
          (0, _util.$debug)('Found a class that isn\'t a react component', superClass);
          return;
        }

        var propTypes = null,
            contextTypes = null;
        // And have type as property annotations
        path.node.body.body.forEach(function (bodyNode) {
          if (bodyNode && bodyNode.key.name === 'props' && bodyNode.typeAnnotation) {
            var annotation = bodyNode.typeAnnotation.typeAnnotation;
            var props = getPropsForTypeAnnotation(annotation);
            if (!props) {
              throw new TypeError('Couldn\'t process \`class { props: This }`');
            }

            propTypes = props;

            return;
          }

          if (bodyNode && bodyNode.key.name === 'context' && bodyNode.typeAnnotation) {
            var _annotation = bodyNode.typeAnnotation.typeAnnotation;
            var context = getPropsForTypeAnnotation(_annotation);
            if (!context) {
              throw new TypeError('Couldn\'t process \`class { context: This }`');
            }

            contextTypes = context;
          }
        });

        // or Component<void, Props, Context>
        var secondSuperParam = getPropsTypeParam(path.node);
        if (secondSuperParam && secondSuperParam.type === 'GenericTypeAnnotation') {
          var typeAliasName = secondSuperParam.id.name;
          if (typeAliasName === 'Object') return;
          var props = internalTypes[typeAliasName] || importedTypes[typeAliasName];
          if (!props) {
            throw new TypeError('Couldn\'t find type "' + typeAliasName + '"');
          }

          propTypes = props;
        }

        var thirdSuperParam = getContextTypeParam(path.node);
        if (thirdSuperParam && thirdSuperParam.type === 'GenericTypeAnnotation') {
          var _typeAliasName = thirdSuperParam.id.name;
          if (_typeAliasName === 'Object') return;
          var _props = internalTypes[_typeAliasName] || importedTypes[_typeAliasName];
          if (!_props) {
            throw new TypeError('Couldn\'t find type "' + _typeAliasName + '"');
          }

          contextTypes = _props;
        }

        annotate(path, propTypes, contextTypes);
      },
      FunctionExpression: function FunctionExpression(path) {
        if (suppress) return;
        return functionVisitor(path);
      },
      FunctionDeclaration: function FunctionDeclaration(path) {
        if (suppress) return;
        return functionVisitor(path);
      },
      ArrowFunctionExpression: function ArrowFunctionExpression(path) {
        if (suppress) return;
        return functionVisitor(path);
      },


      // See issue:
      /**
         * Processes exported type aliases.
         *
         * This function also adds something to the AST directly, instead
         * of invoking annotate.
         *
         * @param path
         * @constructor
         */
      ExportNamedDeclaration: function ExportNamedDeclaration(path) {
        if (suppress) return;
        var node = path.node;


        if (!node.declaration || node.declaration.type !== 'TypeAlias') {
          return;
        }

        var declarationObject = node.declaration.right;

        var name = node.declaration.id.name;
        var propTypes = convertNodeToPropTypes(declarationObject);
        internalTypes[name] = propTypes;

        var propTypesAst = (0, _makePropTypesAst.makePropTypesAstForExport)(propTypes);

        // create a variable for reuse
        var exportedName = (0, _util.getExportNameForType)(name);
        exportedTypes[name] = exportedName;
        var variableDeclarationAst = t.variableDeclaration('var', [t.variableDeclarator(t.identifier(exportedName), wrapInDceCheck(propTypesAst))]);
        path.insertBefore(variableDeclarationAst);

        if (!omitRuntimeTypeExport) {
          // add the variable to the exports
          var exportAst = t.expressionStatement(t.callExpression(t.memberExpression(t.identifier('Object'), t.identifier('defineProperty')), [t.identifier('exports'), t.stringLiteral((0, _util.getExportNameForType)(name)), t.objectExpression([t.objectProperty(t.identifier('value'), t.identifier(exportedName)), t.objectProperty(t.identifier('configurable'), t.booleanLiteral(true))])]));
          var exportsDefinedCondition = t.binaryExpression('!==', t.unaryExpression('typeof', t.identifier('exports')), t.stringLiteral('undefined'));

          var ifCond = exportsDefinedCondition;
          if (opts.deadCode) {
            var dceConditional = t.unaryExpression('!', getDcePredicate());
            ifCond = t.logicalExpression('&&', dceConditional, ifCond);
          }

          var conditionalExportsAst = t.ifStatement(ifCond, exportAst);
          path.insertAfter(conditionalExportsAst);
        }
      },
      ImportDeclaration: function ImportDeclaration(path) {
        if (suppress) return;
        var node = path.node;

        // https://github.com/brigand/babel-plugin-flow-react-proptypes/issues/62
        // if (node.source.value[0] !== '.') {
        //   return;
        // }

        if (node.importKind === 'type') {
          node.specifiers.forEach(function (specifier) {
            var typeName = specifier.type === 'ImportDefaultSpecifier' ? specifier.local.name : specifier.imported.name;
            // Store the name the type so we can use it later. We do
            // mark it as importedTypes because we do handle these
            // differently than internalTypes.
            // imported types are basically realized as imports;
            // because we can be somewhat sure that we generated
            // the proper exported propTypes in the imported file
            // Later, we will check importedTypes to determine if
            // we want to put this as a 'raw' type in our internal
            // representation
            importedTypes[typeName] = (0, _util.getExportNameForType)(typeName);

            // https://github.com/brigand/babel-plugin-flow-react-proptypes/issues/129
            if (node.source.value === 'react' && typeName === 'ComponentType') {
              var ast = t.variableDeclaration('var', [t.variableDeclarator(t.identifier((0, _util.getExportNameForType)(typeName)), t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral('prop-types')]), t.identifier('func')))]);
              path.insertAfter(ast);
              return;
            }

            var variableDeclarationAst = t.variableDeclaration('var', [t.variableDeclarator(
            // TODO: use local import name?
            t.identifier((0, _util.getExportNameForType)(typeName)), t.logicalExpression('||', t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral(node.source.value)]), t.identifier((0, _util.getExportNameForType)(typeName))), t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral('prop-types')]), t.identifier('any'))))]);
            path.insertAfter(variableDeclarationAst);
          });
        }
      }
    }
  };
};

function getPropsTypeParam(node) {
  if (!node) return null;
  if (!node.superTypeParameters) return null;
  var superTypes = node.superTypeParameters;
  if (superTypes.params.length === 2) {
    return superTypes.params[0];
  } else if (superTypes.params.length === 3) {
    return superTypes.params[0];
  } else if (superTypes.params.length === 1) {
    return superTypes.params[0];
  }
  return null;
}

function getContextTypeParam(node) {
  if (!node) return null;
  if (!node.superTypeParameters) return null;
  var superTypes = node.superTypeParameters;
  if (superTypes.params.length === 3) {
    return superTypes.params[2];
  }
  return null;
}
